
create database filetab_demo
on primary 
  (name = filetab_demo_file, filename = N'C:\Files\FileTab_Demo\FileTab_Demo_File.mdf' ),
filegroup FSStorage contains filestream 
  ( name = filetab_demo_fs_file, filename = N'C:\Files\FileTab_Demo\FileTab_Demo_FS' )
with filestream ( 
    non_transacted_access = full,
    directory_name = N'FileTab_Demo' )
go

use filetab_demo
go

-- create filetable
create table filetab_demo..Documents as FileTable 
with (filetable_directory = N'Documents')
go

select * from Documents;

-- now, add some documents with Win32
-- or drag and drop

-- fails
delete Documents where name = 'SQL Server';

select * from Documents where name = 'SomeText2.txt';

-- works
delete Documents where name = 'SomeText2.txt';

-- path_locator and parent_path_locator are hierarchyids
-- get the friendly names
select 
  name,
  path_locator,
  file_stream.GetFileNamespacePath() as FilePath, -- equals GetFileNamespacePath(0)
  file_stream.GetFileNamespacePath(1) as FullFilePath,
  FileTableRootPath() as RootPath,
  FileTableRootPath('dbo.Documents') as [RootPath Of This Table],
  --FileTableRootPath('dbo.Documents',0),  -- NETBIOS name
  --FileTableRootPath('dbo.Documents',1),
  --FileTableRootPath('dbo.Documents',2),

  GetPathLocator(file_stream.GetFileNamespacePath(1)) as PathLocator,

  file_stream.PathName() as FileStreamPath  -- FileStream Path Name
  --file_stream.PathName(0, 0) -- NETBIOS name, retrieve VNN
  --file_stream.PathName(0, 1) -- NETBIOS name, retrieve computer name
from Documents

DECLARE @fullpath nvarchar(260);
SELECT @fullpath = FileTableRootPath() + file_stream.GetFileNamespacePath()
FROM dbo.Documents
WHERE name = 'SomeText.txt';

SELECT @fullpath
-- This gets a filetable pathlocator from a path name
SELECT GetPathLocator(@fullpath) as PathLocatorHierarchyid;

SELECT path_locator
FROM dbo.Documents
WHERE name = 'SomeText.txt';
go

-- try it with a directory
DECLARE @fullpath nvarchar(260);
SELECT @fullpath = FileTableRootPath() + file_stream.GetFileNamespacePath()
FROM dbo.Documents
WHERE name = 'SQL Server';

SELECT @fullpath
-- This gets a filetable pathlocator from a path name
SELECT GetPathLocator(@fullpath) as PathLocatorHierarchyid;

SELECT path_locator
FROM dbo.Documents
WHERE name = 'SQL Server';

-- use the path_locator and parent_path_locator as hierarchyid
-- Level Number
SELECT path_locator.GetLevel() as Level, 
	file_stream.GetFileNamespacePath() as NameSpacePath, 
	--path_locator.ToString() as Path,
	name
FROM dbo.Documents p 
go

-- Level Number of table is 0. Table PathLocator is root
SELECT FileTableRootPath('dbo.Documents') as RootPath,
       GetPathLocator(FileTableRootPath('dbo.Documents')) as Path, 
       GetPathLocator(FileTableRootPath('dbo.Documents')).GetLevel() as Level

-- an interesting way to get all direct children of the top-level SQL Server node
SELECT p.name as ParentName, c.name as ChildName, c.is_directory,
	c.file_stream.GetFileNamespacePath() as ChildPath
FROM dbo.Documents p 
JOIN dbo.Documents c
ON p.name = 'SQL Server'
AND p.path_locator.GetLevel() = 1
AND c.path_locator.GetAncestor(1) = p.path_locator

-- or 
SELECT p.name as ParentName, c.name as ChildName, c.is_directory,
	c.file_stream.GetFileNamespacePath() as ChildPath
FROM dbo.Documents p 
JOIN dbo.Documents c
ON p.name = 'SQL Server'
AND p.path_locator.GetLevel() = 1
AND c.parent_path_locator = p.path_locator

-- child of the top-level SQL Server node, any 
SELECT p.name as ParentName, c.name as ChildName, c.is_directory,
	c.file_stream.GetFileNamespacePath() as ChildPath
FROM dbo.Documents p 
JOIN dbo.Documents c
ON p.name = 'SQL Server'
AND p.path_locator.GetLevel() = 1
AND c.path_locator.IsDescendantOf(p.path_locator) = 1

-- parents of a specific node
SELECT p.name as ParentName, c.name as ChildName, p.is_directory,
	p.file_stream.GetFileNamespacePath() as ParentPath
FROM dbo.Documents p 
JOIN dbo.Documents c
ON c.name = 'SQLSubdirSomeText.txt'
AND c.path_locator.IsDescendantOf(p.path_locator) = 1

select * from documents

insert dbo.Documents(Name, file_stream) values('SyntheticFile.txt', 0x)

declare @path hierarchyid;
declare @pathstring nvarchar(max);
declare @newpath nvarchar(max);

SELECT @pathstring = path_locator.ToString() from documents where name = 'SQL Server' AND path_locator.GetLevel() = 1
SET @newpath = @pathstring +  convert(varchar(20), convert(bigint, substring(convert(binary(16), newid()), 1, 6))) + '.' 
                           +  convert(varchar(20), convert(bigint, substring(convert(binary(16), newid()), 7, 6))) + '.' 
						   +  convert(varchar(20), convert(bigint, substring(convert(binary(16), newid()), 13, 4))) + '/';

--insert dbo.Documents(Name, path_locator, file_stream) values('SubSyntheticFile.txt', @pathstring + '1/', 0x)
insert dbo.Documents(Name, path_locator, file_stream) values('SubSyntheticFile2.txt', @newpath, 0x)

-- not allowed
update dbo.Documents set parent_path_locator = NULL
where name = 'SubSyntheticFile2.txt'
go

-- new synthetic hierarchyid
declare @pathstring nvarchar(max);
declare @newpath nvarchar(max);
SELECT @pathstring = path_locator.ToString() from documents where name = 'SQLServerSubdir' AND path_locator.GetLevel() = 2;
SET @newpath = @pathstring +  convert(varchar(20), convert(bigint, substring(convert(binary(16), newid()), 1, 6))) + '.' 
                           +  convert(varchar(20), convert(bigint, substring(convert(binary(16), newid()), 7, 6))) + '.' 
						   +  convert(varchar(20), convert(bigint, substring(convert(binary(16), newid()), 13, 4))) + '/';
update dbo.Documents set path_locator = @newpath
where name = 'SubSyntheticFile2.txt'
go

-- OK, now use Reparent to move it back
DECLARE @oldpath hierarchyid, @newpath hierarchyid 
SELECT @oldpath = path_locator from documents where name = 'SQLServerSubdir' AND path_locator.GetLevel() = 2;
SELECT @newpath = path_locator from documents where name = 'SQL Server' AND path_locator.GetLevel() = 1;
update dbo.Documents set path_locator = path_locator.GetReparentedValue(@oldpath, @newpath)
where name = 'SubSyntheticFile2.txt';
go

-- Here's the BOL proposed use for GetPathLocator():

-- you'd use path locator to hook up your additional metadata table
-- Add a path locator column to the PhotoMetadata table.
ALTER TABLE PhotoMetadata ADD pathlocator hierarchyid;

-- Get the root path of the Photo directory on the File Server.
DECLARE @UNCPathRoot varchar(100) = '\\RemoteShare\Photographs';

-- Get the root path of the FileTable.
DECLARE @FileTableRoot varchar(1000);
SELECT @FileTableRoot = FileTableRootPath('dbo.PhotoTable');

-- This is for moving documents from a UNC path to filetable
-- Where another table has a reference to the UNC path
-- Update the PhotoMetadata table.
-- Replace the File Server UNC path with the FileTable path.
UPDATE PhotoMetadata
    SET UNCPath = REPLACE(UNCPath, @UNCPathRoot, @FileTableRoot);

-- Update the pathlocator column to contain the pathlocator IDs from the FileTable.
UPDATE PhotoMetadata
    SET pathlocator = GetPathLocator(UNCPath);

-- Additional functions found by intellisence
select GENDBNAMEFROMPATH('Path') -- ?

-- can't use newid in a function
CREATE FUNCTION random_hierarchyid(@parent varchar(max))
returns hierarchyid
as
begin
return
(convert(hierarchyid, @parent +  convert(varchar(20), convert(bigint, substring(convert(binary(16), newid()), 1, 6))) + '.' +     convert(varchar(20), convert(bigint, substring(convert(binary(16), newid()), 7, 6))) + '.' +     convert(varchar(20), convert(bigint, substring(convert(binary(16), newid()), 13, 4))) + '/'))
end

